import java.util.Scanner;
import javax.swing.*;
public class SudokuBoard{
    private boolean diagnosticOn;
    private boolean bruteForce;
    private int[][] board;//goes in order [x][y], from top left [0][0] to bottom right [8][8]
    private boolean[][][] can;
    private boolean[][][] cannot;//goes in order [x][y][value], from top left 0 [0][0][0] to bottom right [8][8][8]
    private boolean progression;
    private boolean first;
    public SudokuBoard(){
        board = new int[9][9];
        can = new boolean[9][9][9];
        cannot = new boolean[9][9][9];
        first = true;
        diagnosticOn = false;
        bruteForce = false;
        diagnosticPrint("Developer diagnostic enabled.");
    }
    public void diagnosticPrint(String s){
        if(diagnosticOn){
            System.out.println(s);
        }
    }
    public void setBruteForce(boolean set){
        bruteForce = set;
    }
    public boolean getBruteForce(){
        return bruteForce;
    }
    public void write(JTable jt){
        if(diagnosticOn){
            System.out.println("Taking information from a JTable.");
        }
        Scanner tableScan;
        for(int i = 0;i<9;i++){
            for(int j = 0;j<9;j++){
                tableScan = new Scanner(jt.getValueAt(j,i).toString());//getValueAt works in the form (row, column), or (y,x), unfortunately
                if(tableScan.hasNextInt()){
                    setCell(i,j,tableScan.nextInt());
                }
                else{
                    board[i][j] = 0;
                }
            }
        }
    }
    public void write(int[][] input){
        for(int i = 0;i<9;i++){
            for(int j = 0;j<9;j++){
                board[i][j] = input[i][j];
            }
        }
    }
    public void setRow(int row,int[] entries){
        for(int i = 0;i<9;i++){
            if(entries[i]>0&&entries[i]<10){
                setCell(i,row,entries[i]);
            }
        }
    }
    public void setRow(int row,String entries){
        Scanner entryReader = new Scanner(entries);
        int[] entryHolder = new int[9];
        for(int i = 0;i<9;i++){
            entryHolder[i] = entryReader.nextInt();
        }
        setRow(row,entryHolder);
        entryReader.close();
    }
    public void setCell(int column, int row, int value){
        board[column][row] = value;
        value--;
        can[column][row][value] = true;
        for(int i = 0;i<9;i++){
            if(i!=value){
                voidCell(column,row,i);
            }
        }
    }
    public boolean isFull(){
        for(int i=0;i<9;i++){
            for(int j = 0;j<9;j++){
                if(board[i][j]<1||board[i][j]>9){
                    return false;
                }
            }
        }
        return true;
    }
    public boolean isSolved(){
        boolean[] cvalues = new boolean[9];
        boolean[] rvalues = new boolean[9];
        boolean[] bvalues = new boolean[9];
        if(!isFull()){
            return false;
        }
        for(int i=0;i<9;i++){
            rvalues = new boolean[9];
            cvalues = new boolean[9];
            bvalues = new boolean[9];
            for(int k = 0;k<9;k++){
                if(!cvalues[(board[i][k]-1)]){
                    cvalues[(board[i][k]-1)] = true;
                }
                else{
                    diagnosticPrint("Board not solved due to column "+i+".");
                    return false;
                }
                if(!rvalues[(board[k][i]-1)]){
                    rvalues[(board[k][i]-1)] = true;
                }
                else{
                    diagnosticPrint("Board not solved due to row "+i+".");
                    return false;
                }
                if(!bvalues[board[(i%3)*3+k%3][(i-i%3)+(k-k%3)/3]-1]){
                    bvalues[board[(i%3)*3+k%3][(i-i%3)+(k-k%3)/3]-1] = true;
                }
                else{
                    diagnosticPrint("Board not solved due to box "+(i%3)+","+((i-i%3)/3)+" with k value "+k);
                    return false;
                }
            }
        }
        diagnosticPrint("Board confirmed solved.");
        return true;
    }
    public void readyLogic(){
        cannot = new boolean[9][9][9];
        first = false;
    }
    public boolean solve(){ 
        iterate();
        progression = false;
        for(int i=0;i<9;i++){
            checkRow(i);
            checkColumn(i);
            checkBox(((i-i%3)/3),(i%3));
        }
        if(isSolved()){
            diagnosticPrint("Board solved successfully, congratulations.");
            first = true;
            return true;
        }
        else{
            if(isFull()){
                diagnosticPrint("Rejected an unsolved board because it was full.");
                return false;
            }
            if(progression){
                diagnosticPrint("Progression made, entering a new solving iteration.");
                return solve();
            }
            else{
                diagnosticPrint("No progression made.");
                if(!bruteForce){
                    diagnosticPrint("Admitting defeat.");
                    return false;
                }
                else{
                    int[][] boardBackup;
                    boolean[][][] logicBackup;
                    boardBackup = board.clone();
                    logicBackup = cannot.clone();
                    for(int v = 0;v<9;v++){
                        if(fillInFirst(v)){
                            if(solve()){
                                return true;
                            }
                            else{
                                board = boardBackup.clone();
                                cannot = logicBackup.clone();
                            }
                        }
                    }
                    return false;
                }
            }
        }
    }
    private boolean fillInFirst(int v){
        for(int i = 0;i<9;i++){
            for(int j = 0;j<9;j++){
                if(!cannot[i][j][v]){
                    diagnosticPrint("Forcing number "+(v+1)+" at coords "+i+","+j);
                    setCell(i,j,(v+1));
                    return true;
                }
            }
        }
        return false;
    }
    private void iterate(){
        diagnosticPrint("Adding numbers to the board.");
        int single;
        for(int i = 0;i<9;i++){
            for(int j = 0;j<9;j++){
                if(board[i][j]<=0||board[i][j]>9){
                    single = -1;
                    for(int k = 0;k<9;k++){
                        if(!cannot[i][j][k]){
                            if(single>=0){
                                single = -1;
                                break;
                            }
                            else{
                                single = k;
                            }
                        }
                    }
                    if(single>=0){
                        setCell(i,j,single+1);
                    }
                }
            }
        }
    }
    private void checkRow(int row){
        for(int i = 0;i<9;i++){
            if(board[i][row]>0&&board[i][row]<=9){
                voidRow(row, board[i][row]-1);
            }
        }
    }
    private void voidRow(int row, int value){
        //diagnosticPrint("Voiding row "+row+" of number "+(value+1));
        for(int i = 0;i<9;i++){
            voidCell(i,row,value);
        }
    }
    private void voidRowMinus(int row, int value, int boxX){
        //diagnosticPrint("Voiding row "+row+" (except box column "+boxX+") of number "+(value+1));
        boxX *= 3;
        for(int i = 0;i<9;i++){
            if(!(i>=boxX&&i<boxX+3)){
                voidCell(i,row,value);
            }
        }
    }
    private void checkColumn(int column){
        for(int i = 0;i<9;i++){
            if(board[column][i]>0&&board[column][i]<=9){
                voidColumn(column, board[column][i]-1);
            }
        }
    }
    private void voidColumn(int column, int value){
        //diagnosticPrint("Voiding column "+column+" of number "+(value+1));
        for(int i = 0;i<9;i++){
            voidCell(column,i,value);
        }
    }
    private void voidColumnMinus(int column, int value, int boxY){
        //diagnosticPrint("Voiding column "+column+" (except box row "+boxY+") of number "+(value+1));
        boxY *= 3;
        for(int i = 0;i<9;i++){
            if(!(i>=boxY&&i<boxY+3)){
                voidCell(column,i,value);
            }
        }
    }
    private boolean boxContains(int x,int y,int number){
        x *= 3;
        y *= 3;
        for(int i = x;i<(x+3);i++){
            for(int j = y;j<(y+3);j++){
                if(board[i][j]==number){
                    return true;
                }
            }
        }
        return false;
    }
    private void checkBox(int x, int y){
        x *= 3;
        y *= 3;
        boolean[] present = new boolean[9];
        int xFind = 0;
        int yFind = 0;
        boolean foundOne;
        boolean foundMore;
        for(int i = x;i<(x+3);i++){
            for(int j = y;j<(y+3);j++){
                if(board[i][j]>0&&board[i][j]<=9){
                    voidBox(x/3,y/3,board[i][j]-1);
                }
            }
        }
        for(int i = 0;i<9;i++){
            if(boxContains(x/3,y/3,i+1)){
                present[i] = true;
            }
        }
        for(int k = 0;k<9;k++){
            if(!present[k]){
                xFind = -1;
                yFind = -1;
                foundOne = false;
                for(int i = x;i<(x+3);i++){
                    for(int j = y;j<(y+3);j++){
                        if(!cannot[i][j][k]){
                            if(!foundOne){
                                xFind = i;
                                yFind = j;
                                foundOne = true;
                            }
                            else{
                                xFind = -1;
                                yFind = -1;
                            }
                        }
                    }
                }
                if(xFind>=0){
                    setCell(xFind,yFind,k+1);
                }
            }
        }
        for(int k = 0;k<9;k++){
            if(!present[k]){
                xFind = -1;
                yFind = -1;
                foundOne = false;
                foundMore = false;
                for(int i = x;i<(x+3);i++){
                    for(int j = y;j<(y+3);j++){
                        if(!cannot[i][j][k]){
                            if(!foundOne&&!foundMore){
                                foundOne = true;
                                xFind = i;
                                yFind = j;
                            }
                            else if(!foundMore){
                                foundMore = true;
                                if(i!=xFind){
                                    xFind = -1;
                                }
                                if(j!=yFind){
                                    yFind = -1;
                                }
                            }
                            else{
                                if(i!=xFind){
                                    xFind = -1;
                                }
                                if(j!=yFind){
                                    yFind = -1;
                                }
                            }
                        }
                    }
                }
                if(foundMore){
                    if(xFind>=0){
                        diagnosticPrint("Inferring that column "+xFind+" has none of number "+(k+1));
                        voidColumnMinus(xFind,k,y/3);
                    }
                    if(yFind>=0){
                        diagnosticPrint("Inferring that row "+yFind+" has none of number "+(k+1));
                        voidRowMinus(yFind,k,x/3);
                    }
                }
            }
        }
    }
    private void voidBox(int x, int y, int value){
        //diagnosticPrint("Voiding box at box coordinate "+x+","+y+" of number "+(value+1));
        x *= 3;
        y *= 3;
        for(int i = x;i<(x+3);i++){
            for(int j = y;j<(y+3);j++){
                voidCell(i,j,value);
            }
        }
    }
    private void voidCell(int x, int y, int value){
        if(cannot[x][y][value] != true){
            if(!progression){diagnosticPrint("Initial progression for this step made.");};
            cannot[x][y][value] = true;
            progression = true;
        }
    }
    private char getCell(int row,int column){
        if(board[column][row]>0&&board[column][row]<=9){
            return (Integer.toString(board[column][row]).charAt(0));
        }
        else{
            return 'X';
        }
    }
    public int[][] toMatrix(){
        return board;
    }
    //deprecated method that fills a 9x9 table with the data
    public void swingPrint(boolean solved,JTable jt){
        diagnosticPrint("Printing board to a JTable on the GUI.");
        //diagnosticPrint("value of cannot[0][0][8] = "+cannot[0][0][8]);
        if(solved){
            for(int i = 0;i<9;i++){
                for(int j = 0;j<9;j++){
                    jt.setValueAt(board[i][j],j,i);
                }
            }
        }
        else{
            for(int i = 0;i<9;i++){
                for(int j = 0;j<9;j++){
                    if(board[j][i]>0&&board[j][i]<=9){
                        jt.setValueAt(board[j][i],i,j);
                    }
                    else{
                        jt.setValueAt("X",i,j);
                    }
                }
            }
        }
    }
    //Deprecated console print
    public void print(){
        for(int i=0;i<9;i++){
            printRow(i);
            if(((i+1)%3)==0&&i<8){
                System.out.println("-----|-----|-----");
            }
        }
    }
    //Deprecated console print
    private void printRow(int row){
        for(int i=0;i<9;i++){
            System.out.print(getCell(row,i));;
            if(((i+1)%3)==0&&i<8){
                System.out.print("|");
            }
            else{
                System.out.print(" ");
            }
        }
        System.out.print("\n");
    }
}